/*****************************************************************************
Copyright 1987, 1988, 1989, 1990, 1991 by Digital Equipment Corp., Maynard, MA

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of Digital not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

*****************************************************************************/
/*
 *
 *  CONTRIBUTORS:
 *
 *      Dick Annicchiarico
 *      Robert Chesler
 *      Dan Coutu
 *      Gene Durso
 *      Marc Evans
 *      Alan Jamison
 *      Mark Henry
 *      Ken Miller
 *
 */
#ifndef lint
static char SCCSID[] = "@(#)XETrapDispatch.c	1.18 - 90/11/12  ";
static char RCSID[] = "$Header$";
#endif

#include "stdio.h"
#include "xtraplib.h"
#include "xtraplibproto.h"

#ifdef FUNCTION_PROTOS
static void XETrapDispatchCB(XETC *tc, XETrapDatum *pdatum)
#else
static void XETrapDispatchCB(tc, pdatum)
    XETC *tc;
    XETrapDatum *pdatum;
#endif
{
    void_function pfunc = NULL;
    BYTE *userp = NULL;

    /* Need to deal with Delta Timestamps here before calling client CB */
    if (XETrapGetTCFlagDeltaTimes(tc))
    {
        CARD32 last_time = XETrapGetTCTime(tc);
        if (!last_time)
        {   /* first one!  Prime it! */
            last_time = pdatum->hdr.timestamp;
        }
        tc->values.last_time = pdatum->hdr.timestamp;   /* no macro! */
        /* absolute delta in case the clock rolls over */
        pdatum->hdr.timestamp = abs(pdatum->hdr.timestamp - last_time);
    }

    /*  Get the user supplied callback function */
    if (XETrapHeaderIsEvent(&pdatum->hdr))
    {
        pfunc = tc->values.evt_cb[pdatum->u.event.u.u.type].func;
        userp = tc->values.evt_cb[pdatum->u.event.u.u.type].data;
    }
    else if (XETrapHeaderIsRequest(&pdatum->hdr))
    {
        pfunc = tc->values.req_cb[pdatum->u.req.reqType].func;
        userp = tc->values.req_cb[pdatum->u.req.reqType].data;
    }

    /* If there is a callback then call it with the data */
    if (pfunc != NULL)
    { 
        (*pfunc)(tc,pdatum,userp); 
    }
}

#ifdef FUNCTION_PROTOS
void XETrapDispatchRead(XETC *tc)
#else
void XETrapDispatchRead(tc)
    XETC *tc;
#endif
{
    static XETrapDatum *pdatum = NULL;
    static int dsize = 0;
    XESTREAM *st = XETrapGetTCStream(tc);

#ifdef vms
    /* False alarm? */
    if (st->u.Q.byte_cnt < XETrapMinPktSize)
    {   /* yup! */
        return;
    }
#endif /* vms */
    /* Do we need memory? */
    if ((pdatum == NULL || dsize == 0) &&
        (pdatum = (XETrapDatum *)XtMalloc((dsize = sizeof(*pdatum)))) == NULL)
    {   /* Then Xt reported an error; should not try to continue */
        return;
    }

    /* Perform the read on the desired stream */
    if (s_read(st,pdatum,XETrapMinPktSize) == XETrapMinPktSize)
    {   /* Is there more data to be read? */
#ifdef DEBUG
        int i;
        char *ptr;
        fprintf(stderr,"DRead Hdr: cnt TS type scrn x y client data(4)");
        fprintf(stderr,"  %d %%x0%x  %d %d %d %d %d %d %d %d\n",
        pdatum->hdr.count,pdatum->hdr.timestamp,pdatum->hdr.type,
        pdatum->hdr.screen,pdatum->hdr.win_x,pdatum->hdr.win_y,
        pdatum->hdr.client,pdatum->u.event.u.u.type,
        pdatum->u.event.u.u.detail,pdatum->u.event.u.u.sequenceNumber);
        ptr = (char *)&(pdatum->u.req);
        fprintf(stderr, "DRead Raw:  ");
        for (i=0; i<32; i++)
        {
            fprintf(stderr, "%x ", *ptr++);
        }
        fprintf(stderr, "\n");
#endif  /* DEBUG */
        if (pdatum->hdr.count > XETrapMinPktSize)
        {   /* Do we need more memory? */
            if (pdatum->hdr.count > (XMaxRequestSize(tc->dpy) * sizeof(CARD32)
                + sizeof(XETrapHeader)))
            {   /* there's bogus data in this field!  Complain and return */
                static char params[12L];
                static int num_params = 1L;
                static char *param_vec = params;
                (void)sprintf(params, "%x", pdatum->hdr.count);
                XtErrorMsg("BadReadLength", "XETrapDispatchRead",
                    "XTrapToolkitError", 
                    "invalid number bytes to be read from XTrap (0x%s)",
                    &param_vec, &num_params);
                return;
            }
            if (dsize < pdatum->hdr.count &&
                (pdatum = (XETrapDatum *)XtRealloc(pdatum,
                    (dsize = pdatum->hdr.count))) == NULL)
            {   /* Then Xt reported an error; should not try to continue */
                return;
            }

            /* Read the additional data from the stream */
            if (s_read(st, ((char *)pdatum + XETrapMinPktSize),
                pdatum->hdr.count - XETrapMinPktSize) != 
                pdatum->hdr.count - XETrapMinPktSize)
            {   /* Then we have a stream read error/exception */
                char *params = "extended pkt data";
                int num_params = 1L;
                XtErrorMsg("CantReadStream", "XETrapDispatchRead",
		    "XTrapToolkitError",
		    "unable to read %s from XTrap", &params, &num_params);
                return;
            }
#ifdef DEBUG
        ptr = (char *)pdatum + XETrapMinPktSize;
        fprintf(stderr, "DRead Raw2:  ");
        for (i=0; i<pdatum->hdr.count - XETrapMinPktSize; i++)
        {
            fprintf(stderr, "%x ", ptr++);
        }
        fprintf(stderr, "\n");
#endif  /* DEBUG */
        }
        XETrapDispatchCB(tc, pdatum);
    }
    else
    {   /* Then we have a stream read error/exception */
        char *params = "minimum pkt size";
        int num_params = 1L;
        XtErrorMsg("CantReadStream", "XETrapDispatchRead", "XTrapToolkitError",
	    "unable to read %s from XTrap", &params, &num_params);
        return;
    }
    return;
}

#ifdef FUNCTION_PROTOS
void XETrapDispatchXLib(XETrapDataEvent *event, XETC *tc)
#else
void XETrapDispatchXLib(event, tc)
    XETrapDataEvent *event;
    XETC   *tc;
#endif
{   
    memcpy(&tc->xbuff[event->idx*sz_EventData], event->data, sz_EventData);

    if (event->detail == XETrapDataLast)
    {
        XETrapDispatchCB(tc, (XETrapDatum *)tc->xbuff);
    }
}
