/*****************************************************************************
Copyright 1987, 1988, 1989, 1990, 1991 by Digital Equipment Corp., Maynard, MA

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of Digital not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

*****************************************************************************/
/*
 *  ABSTRACT:
 *
 *      This module is the platform-specific but conditionally independent
 *      code for the XTrap extension (usually I/O or platform setup).
 *      This is shared code and is subject to change only by team approval.
 *
 *  CONTRIBUTORS:
 *
 *      Dick Annicchiarico
 *      Robert Chesler
 *      Gene Durso
 *      Marc Evans
 *      Alan Jamison
 *      Mark Henry
 *      Ken Miller
 *
 */
#ifndef lint
static char SCCSID[] = "@(#)xtrapddmi.c	1.24 - 90/09/18  ";
static char RCSID[] = "$Header$";
#endif
#include "stdio.h"
#ifdef PC
# include "fcntl.h"
# include "io.h"
# define O_NDELAY 0L
#else
# include "file.h"
#endif
#include "types.h"
#include "errno.h"

#define NEED_REPLIES
#define NEED_EVENTS
#include "X.h"        /* From library include environment */
#include "input.h"    /* From server include env. (must be before Xlib.h!) */
#ifdef PC
# include "scrninst.h"
#else
# include "scrnintstr.h"
#endif

/* 
#include "time.h"
#include "param.h"
#include "uio.h" */   /* for vectored writes -- also needs types.h */

#include "xtrapdi.h"
#include "xtrapddmi.h"
#include "xtrapproto.h"

extern int_function     XETrapIoConf[MaxFamilies];
extern int XETrapErrorBase;
extern xXTrapGetAvailReply XETrap_avail;
extern DevicePtr XETrapKbdDev;
extern DevicePtr XETrapPtrDev;

#if defined TCPCONN || (defined DNETCONN && !defined vms)
# include "socket.h"
#endif

#ifdef TCPCONN
# include "in.h"
# include "netdb.h"
#endif

#if defined DNETCONN && !defined vms
# include "dn.h"
#endif

#ifdef vms
# include "iodef.h"
#endif

/*
 *  DESCRIPTION:
 *
 *      This function performs the platform specific setup for server
 *      extension implementations.
 */
void XETrapPlatformSetup()
{
#ifdef TCPCONN
    XETrapIoConf[FamilyInternet] = XETrapConfigInternet;
    XETrap_avail.data.xport_msk_out |= BitInLong(FamilyInternet);
#endif  /* TCPCONN */
#ifdef DNETCONN
    XETrapIoConf[FamilyDECnet]   = XETrapConfigDECnet;
    XETrap_avail.data.xport_msk_out |= BitInLong(FamilyDECnet);
#if (defined vms) && (!defined _XINPUT)
    XETrap_avail.data.xport_msk_in |= BitInLong(FamilyDECnet);
#endif
#endif  /* DNETCONN */
#ifdef vms
    XETrapIoConf[FamilyMbx]      = XETrapConfigMbx;
    XETrap_avail.data.xport_msk_out |= BitInLong(FamilyMbx);
#ifndef _XINPUT
    XETrap_avail.data.xport_msk_in |= BitInLong(FamilyMbx);
#endif
#endif /* vms */
}


#if !defined _XINPUT && !defined vms
/*
 *  DESCRIPTION:
 *
 *  This routine processes the simulation of some input event.
 *
 */
#ifdef FUNCTION_PROTOS
int XETrapSimulateXEvent(register xXTrapInputReq *request,
    register ClientPtr client)
#else
int XETrapSimulateXEvent(request, client)
    register xXTrapInputReq *request;
    register ClientPtr client;
#endif
{
    ScreenPtr pScr;
    int status = Success;
    xEvent xev;
    register int x = request->input.x;
    register int y = request->input.y;
    DevicePtr keydev = LookupKeyboardDevice();
    DevicePtr ptrdev = LookupPointerDevice();

    if (request->input.screen < screenInfo.numScreens)
    {
#ifdef X11R4
        pScr = screenInfo.screens[request->input.screen];
#else
        pScr = &screenInfo.screen[request->input.screen];
#endif
    }
    else
    {   /* Trying to play bogus events to this WS! */
#ifdef VERBOSE
        ErrorF("%s:  Trying to send events to screen %d!\n", XTrapExtName,
            request->input.screen);
#endif
        status = XETrapErrorBase + BadScreen;
    }
    /* Fill in the event structure with the information
     * Note:  root, event, child, eventX, eventY, state, and sameScreen
     *        are all updated by FixUpEventFromWindow() when the events
     *        are delivered via DeliverDeviceEvents() or whatever.  XTrap
     *        needs to only concern itself with type, detail, time, rootX, 
     *        and rootY.
     */
    if (status == Success)
    {
        xev.u.u.type   = request->input.type;
        xev.u.u.detail = request->input.detail;
        xev.u.keyButtonPointer.time   = GetTimeInMillis();
        xev.u.keyButtonPointer.rootX = x;
        xev.u.keyButtonPointer.rootY = y;

        if (request->input.type == MotionNotify)
        {   /* Set new cursor position on screen */
            XETrap_avail.data.cur_x = x;
            XETrap_avail.data.cur_y = y;
            if (!(*pScr->SetCursorPosition)(pScr, x, y, xTrue))
            {
                status = BadImplementation;
            }
        }
    }
    if (status == Success)
    {
        switch(request->input.type)
        {   /* Now process the event appropriately */
            case KeyPress:
            case KeyRelease:
#ifndef X11R4
                (*XETrapKbdDev->processInputProc)(&xev,keydev);
#else
                (*XETrapKbdDev->realInputProc)(&xev,keydev, 1L);
#endif
                break;
            case MotionNotify:
            case ButtonPress:
            case ButtonRelease:
#ifndef X11R4
                (*XETrapPtrDev->processInputProc)(&xev,ptrdev);
#else
                (*XETrapPtrDev->realInputProc)(&xev,ptrdev, 1L);
#endif
                break;
            default:
                status = BadValue;
                break;
        }
    }
    return(status);
}
#endif

/*
 *  DESCRIPTION:
 *
 *      This function performs the transport specific functions for writing
 *      data back to an XTrap client via a generic File Descriptor.
 *
 */
#ifdef FUNCTION_PROTOS
int XETrapWriteFd(XETrapEnv *penv, BYTE *data, CARD32 nbytes)
#else
int XETrapWriteFd(penv, data, nbytes)
    XETrapEnv *penv;
    BYTE      *data;
    CARD32    nbytes;
#endif
{
    return(write(penv->write.id.fd, data, nbytes));
}

#ifdef FUNCTION_PROTOS
int XETrapCloseFd(XETrapID id)
#else
int XETrapCloseFd(id)
    XETrapID id;
#endif
{
    return(close(id.fd));
}

/*
 *  DESCRIPTION:
 *
 *      This function performs the transport family specific operations to
 *      connect to the client application for Internet communication. It
 *      has been written assuming a standard BSD4.3 networking environment,
 *      which may not necessarily be the case (Wollongong maybe?). Any
 *      platform specific changes needed, should be #ifdefed for the specific
 *      environment.
 */
#ifdef TCPCONN
#ifdef FUNCTION_PROTOS
int XETrapConfigInternet(xXTrapConfigReq *request, XETrapEnv *penv)
#else
int XETrapConfigInternet(request,penv)
    xXTrapConfigReq *request;
    XETrapEnv *penv;
#endif
{
    int status = Success;
    char *host = (char *)(request->config.address);
    struct sockaddr_in insock;
    struct hostent *host_ptr;
    short port;
    int fd;
    
    /* 
     * Get the port number which is a null-terminated ascii string beyond the
     * host 
     */
    port = atoi(&host[strlen(host)+1L]);

    /* Get a host address reference */
    if ((insock.sin_addr.s_addr = inet_addr(host)) == -1L)
    {
        if (((host_ptr = gethostbyname(host)) != NULL) &&
        (host_ptr->h_addrtype == AF_INET))
        {
            /* Set up the address to be used in socket calls */
            (void)memcpy(&insock.sin_addr,host_ptr->h_addr,
            sizeof(insock.sin_addr));
        }
        else
        {   /* Send the error to the client */
            status = XETrapErrorBase + BadHostname;
        }
    }
    if (status == Success)
    {
        insock.sin_family = AF_INET;
        insock.sin_port   = port;

        /* Connect up with the client program */
        if (((fd = socket(AF_INET,SOCK_STREAM,0L)) >= 0L) &&
            (connect(fd,&insock,sizeof(insock)) >= 0L))
        {
            /* Based on reading/writing, set up the environment */
#ifndef _XINPUT
            if (BitIsTrue(penv->cur.data.config.flags.data,XETrapTransIn))
            {   /* Then we are reading */
                status = BadImplementation;
            }
#endif
            if (BitIsTrue(penv->cur.data.config.flags.data,XETrapTransOut))
            {   /* Then we are writing */
                penv->write.io        = XETrapWriteFd;
                penv->write.close     = XETrapCloseFd;
                penv->write.id.fd     = fd;
                penv->write.transport = FamilyInternet;
            }
        }
        else
        {   /* Send the error to the client */
            status = XETrapErrorBase + BadSocket;
        }
    }
    else
    {   /* Send the error to the client */
        status = XETrapErrorBase + BadHostname;
    }
    return(status);
}
#endif /* TCPCONN */

/*
 *  DESCRIPTION:
 *
 *      This function performs the transport family specific operations to
 *      connect to the client application for DECnet communication. It
 *      has been written assuming a socket based networking environment,
 *      which may not necessarily be the case (VMS definitely). Any platform
 *      specific changes needed, should be #ifdef'd for the specific
 *      environment.
 */
#ifdef DNETCONN
#ifdef FUNCTION_PROTOS
int XETrapConfigDECnet(xXTrapConfigReq *request, XETrapEnv *penv)
#else
int XETrapConfigDECnet(request,penv)
    xXTrapConfigReq *request;
    XETrapEnv *penv;
#endif
{
#ifdef vms
    char buffer[256]; /* This is real arbitrary */
    short chan;
#endif
    int status = Success;
    char *host = (char *)(request->config.address);
    char *object = &host[strlen(host)+1L];
    int fd;

#ifndef vms
    /* Connect to the client program */
    if ((fd = dnet_conn(host,object,SOCK_STREAM,NULL,NULL,NULL,NULL)) >= 0L)
#else /* vms */
    /* Build the name string */
    sprintf(buffer,"%s::\"0=%s\"",host,object);

    /* Connect to the client program */
    if ((chan = XETrapInitDECnetVMS(buffer, penv)) != -1L)
#endif /* vms */
    {   /* Based on reading/writing, set up the environment */
#ifndef _XINPUT
        if (BitIsTrue(penv->cur.data.config.flags.data,XETrapTransIn))
        {   /* Then we are reading */
#ifndef vms
            status = BadImplementation;
#else /* vms */
            SimInit();          /* Initialize SimInput */
            penv->read.io        = XETrapReadChan;
            penv->read.close     = XETrapCloseDECnetVMS;
            penv->read.id.chan   = chan;
            penv->read.mode      = IO$_READVBLK;
            /* Now post a read for incoming requests */
            if ((*penv->read.io)(penv) == 0L)
            {
                status = XETrapErrorBase + BadIO;
            }
            penv->read.transport = FamilyDECnet;
#endif /* vms */
        }
#endif /* _XINPUT */
        if (BitIsTrue(penv->cur.data.config.flags.data,XETrapTransOut))
        {   /* Then we are writing */
            penv->write.transport = FamilyDECnet;
#ifndef vms
            penv->write.io        = XETrapWriteFd;
            penv->write.close     = XETrapCloseFd;
            penv->write.id.fd     = fd;
#else /* vms */
            penv->write.io        = XETrapWriteChan;
            penv->write.close     = XETrapCloseDECnetVMS;
            penv->write.id.chan   = chan;
            penv->write.mode      = IO$_WRITEVBLK;
#endif /* vms */
        }
    }
    else
    {
        status = XETrapErrorBase + BadHostname;
    }
    return(status);
}
#endif /* DNETCONN */


